@@ -37,26 +37,28 @@
   "Defines which clients are allowed access to the various CA endpoints.
    Each endpoint has a sub-section containing the client whitelist.
    Currently we only control access to the certificate_status(es) endpoints."
-  {:certificate-status {:client-whitelist [schema/Str]}})
+  {:certificate-status
+   {(schema/optional-key :authorization-required) schema/Bool
+    :client-whitelist                             [schema/Str]}})
 
 (def CaSettings
   "Settings from Puppet that are necessary for CA initialization
    and request handling during normal Puppet operation.
    Most of these are Puppet configuration settings."
-  {:access-control        AccessControl
-   :allow-duplicate-certs schema/Bool
-   :autosign              (schema/either schema/Str schema/Bool)
-   :cacert                schema/Str
-   :cacrl                 schema/Str
-   :cakey                 schema/Str
-   :capub                 schema/Str
-   :ca-name               schema/Str
-   :ca-ttl                schema/Int
-   :cert-inventory        schema/Str
-   :csrdir                schema/Str
-   :ruby-load-path        [schema/Str]
-   :signeddir             schema/Str
-   :serial                schema/Str})
+  {:access-control           AccessControl
+   :allow-duplicate-certs    schema/Bool
+   :autosign                 (schema/either schema/Str schema/Bool)
+   :cacert                   schema/Str
+   :cacrl                    schema/Str
+   :cakey                    schema/Str
+   :capub                    schema/Str
+   :ca-name                  schema/Str
+   :ca-ttl                   schema/Int
+   :cert-inventory           schema/Str
+   :csrdir                   schema/Str
+   :ruby-load-path           [schema/Str]
+   :signeddir                schema/Str
+   :serial                   schema/Str})
 
 (def DesiredCertificateState
   "The pair of states that may be submitted to the certificate
@@ -120,6 +122,14 @@
   "Standard value applied to the Netscape Comment extension for certificates"
   "Puppet Server Internal Certificate")
 
+(def required-master-files
+  "The set of SSL files that are required on the master."
+  [:hostprivkey :hostcert])
+
+(def required-ca-files
+  "The set of SSL related files that are required on the CA."
+  [:cacert :cacrl :cakey :cert-inventory :serial])
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Internal
 
@@ -179,17 +189,20 @@
     (IOUtils/copy input-stream os)
     (.toByteArray os)))
 
-(defn partial-state-error
+(schema/defn partial-state-error :- Exception
   "Construct an exception appropriate for the end-user to signify that there
-   are missing SSL files and the CA cannot start until action is taken."
-  [found-files missing-files]
+   are missing SSL files and the master or CA cannot start until action is taken."
+  [master-or-ca :- schema/Str
+   found-files :- [schema/Str]
+   missing-files :- [schema/Str]]
   (IllegalStateException.
    (format
-    (str "Cannot initialize CA with partial state; need all files or none.\n"
+    (str "Cannot initialize %s with partial state; need all files or none.\n"
          "Found:\n"
          "%s\n"
          "Missing:\n"
          "%s\n")
+    master-or-ca
     (str/join "\n" found-files)
     (str/join "\n" missing-files))))
 
@@ -326,7 +339,7 @@
    (utils/subject-key-identifier
      ca-public-key false)])
 
-(schema/defn initialize-ca!
+(schema/defn generate-ssl-files!
   "Given the CA settings, generate and write to disk all of the necessary
   SSL files for the CA. Any existing files will be replaced."
   [ca-settings :- CaSettings
@@ -495,53 +508,91 @@
                        (utils/subject-key-identifier
                          master-public-key false)]]
     (when alt-names-ext
-          (validate-dns-alt-names! alt-names-ext))
+      (validate-dns-alt-names! alt-names-ext))
     (when csr-attr-exts
-          (validate-extensions! csr-attr-exts))
-    (filter (complement nil?)
-            (concat base-ext-list [alt-names-ext] csr-attr-exts))))
+      (validate-extensions! csr-attr-exts))
+    (remove nil? (concat base-ext-list [alt-names-ext] csr-attr-exts))))
 
-(schema/defn initialize-master!
-  "Given the SSL directory file paths, master certname, and CA information,
-  generate and write to disk all of the necessary SSL files for the master.
-  Any existing files will be replaced."
+(schema/defn generate-master-ssl-files!
+  "Given master configuration settings, certname, and CA settings,
+   generate and write to disk all of the necessary SSL files for
+   the master. Any existing files will be replaced."
   [settings :- MasterSettings
-   master-certname :- schema/Str
-   ca-private-key :- (schema/pred utils/private-key?)
-   ca-public-key :- (schema/pred utils/public-key?)
-   ca-cert :- Certificate
-   keylength :- schema/Int
-   serial-file :- schema/Str
-   inventory-file :- schema/Str
-   signeddir :- schema/Str
-   ca-ttl :- schema/Int]
+   certname :- schema/Str
+   ca-settings :- CaSettings
+   keylength :- schema/Int]
   (log/debug (str "Initializing SSL for the Master; settings:\n"
                   (ks/pprint-to-string settings)))
   (create-parent-directories! (vals (settings->ssldir-paths settings)))
   (-> settings :certdir fs/file ks/mkdirs!)
   (-> settings :requestdir fs/file ks/mkdirs!)
-  (let [keypair      (utils/generate-key-pair keylength)
-        public-key   (utils/get-public-key keypair)
-        extensions   (create-master-extensions master-certname
+  (let [ca-cert        (utils/pem->cert (:cacert ca-settings))
+        ca-public-key  (.getPublicKey ca-cert)
+        ca-private-key (utils/pem->private-key (:cakey ca-settings))
+        next-serial    (next-serial-number! (:serial ca-settings))
+        keypair        (utils/generate-key-pair keylength)
+        public-key     (utils/get-public-key keypair)
+        extensions     (create-master-extensions certname
+                                                 public-key
+                                                 ca-public-key
+                                                 settings)
+        private-key    (utils/get-private-key keypair)
+        x500-name      (utils/cn certname)
+        validity       (cert-validity-dates (:ca-ttl ca-settings))
+        hostcert       (utils/sign-certificate (get-subject ca-cert)
+                                               ca-private-key
+                                               next-serial
+                                               (:not-before validity)
+                                               (:not-after validity)
+                                               x500-name
                                                public-key
-                                               ca-public-key
-                                               settings)
-        private-key  (utils/get-private-key keypair)
-        x500-name    (utils/cn master-certname)
-        validity     (cert-validity-dates ca-ttl)
-        hostcert     (utils/sign-certificate (get-subject ca-cert)
-                                             ca-private-key
-                                             (next-serial-number! serial-file)
-                                             (:not-before validity)
-                                             (:not-after validity)
-                                             x500-name public-key
-                                             extensions)]
-    (write-cert-to-inventory! hostcert inventory-file)
+                                               extensions)]
+    (write-cert-to-inventory! hostcert (:cert-inventory ca-settings))
     (utils/key->pem! public-key (:hostpubkey settings))
     (utils/key->pem! private-key (:hostprivkey settings))
     (utils/cert->pem! hostcert (:hostcert settings))
-    (utils/cert->pem! hostcert (path-to-cert signeddir master-certname))
-    (utils/cert->pem! ca-cert (:localcacert settings))))
+    (utils/cert->pem! hostcert
+                      (path-to-cert (:signeddir ca-settings) certname))))
+
+(schema/defn ^:always-validate initialize-master-ssl!
+  "Given configuration settings, certname, and CA settings, ensure all
+   necessary SSL files exist on disk by regenerating all of them if any
+   are found to be missing."
+  ([settings certname ca-settings]
+     (initialize-master-ssl! settings certname ca-settings utils/default-key-length))
+  ([settings :- MasterSettings
+    certname :- schema/Str
+    ca-settings :- CaSettings
+    keylength :- schema/Int]
+     (let [required-files (-> (settings->ssldir-paths settings)
+                              (select-keys required-master-files)
+                              (vals))]
+       (if (every? fs/exists? required-files)
+         (log/info "Master already initialized for SSL")
+         (let [{found   true
+                missing false} (group-by fs/exists? required-files)]
+           (if (= required-files missing)
+             (generate-master-ssl-files! settings certname ca-settings keylength)
+             (throw (partial-state-error "master" found missing))))))))
+
+(schema/defn ^:always-validate retrieve-ca-cert!
+  "Given configuration settings and CA settings, ensure a local copy of the CA
+  cert is available on disk.  cacert is the base CA cert file to copy from and
+  localcacert is where that the CA cert file should be copied to."
+  ([cacert :- schema/Str
+    localcacert :- schema/Str]
+   (if (fs/exists? cacert)
+     (do
+       (ks/mkdirs! (fs/parent localcacert))
+       (fs/copy cacert localcacert))
+     (if-not (fs/exists? localcacert)
+       (throw (IllegalStateException.
+                (str ":localcacert ("
+                     localcacert
+                     ") could not "
+                     "be found and no file at :cacert ("
+                     cacert
+                     ") to copy it from")))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Autosign
@@ -832,42 +883,35 @@
   [cacrl :- schema/Str]
   (slurp cacrl))
 
+(schema/defn ensure-directories-exist!
+  "Create any directories used by the CA if they don't already exist."
+  [settings :- CaSettings]
+  (doseq [dir [:csrdir :signeddir]]
+    (let [path (get settings dir)]
+      (when-not (fs/exists? path)
+        (ks/mkdirs! path)))))
+
 (schema/defn ^:always-validate
   initialize!
-  "Given the CA settings, master file paths, and the master's certname,
-   prepare all necessary SSL files for the master and CA.
-   If all of the necessary SSL files exist, new ones will not be generated.
-   If only some are found (but others are missing), an exception is thrown."
-  ([ca-settings master-file-paths master-certname]
-    (initialize! ca-settings
-                 master-file-paths
-                 master-certname
-                 utils/default-key-length))
-  ([ca-settings :- CaSettings
-    master-settings :- MasterSettings
-    master-certname :- schema/Str
+  "Given the CA configuration settings and an optional keylength,
+   ensure that all required SSL files exist. If all files exist,
+   new ones will not be generated. If only some are found
+   (but others are missing), an exception is thrown."
+  ([settings]
+     (initialize! settings utils/default-key-length))
+  ([settings :- CaSettings
     keylength :- schema/Int]
-    (let [required-ca-files     (vals (settings->cadir-paths ca-settings))
-          required-master-files (vals (settings->ssldir-paths master-settings))]
-      (if (every? fs/exists? required-ca-files)
-        (log/info "CA already initialized for SSL")
-        (let [{found   true
-               missing false} (group-by fs/exists? required-ca-files)]
-          (if (= required-ca-files missing)
-            (initialize-ca! ca-settings keylength)
-            (throw (partial-state-error found missing)))))
-      (if (every? fs/exists? required-master-files)
-        (log/info "Master already initialized for SSL")
-        (initialize-master! master-settings
-                            master-certname
-                            (utils/pem->private-key (:cakey ca-settings))
-                            (utils/pem->public-key (:capub ca-settings))
-                            (utils/pem->cert (:cacert ca-settings))
-                            keylength
-                            (:serial ca-settings)
-                            (:cert-inventory ca-settings)
-                            (:signeddir ca-settings)
-                            (:ca-ttl ca-settings))))))
+     (ensure-directories-exist! settings)
+     (let [required-files (-> (settings->cadir-paths settings)
+                              (select-keys required-ca-files)
+                              (vals))]
+       (if (every? fs/exists? required-files)
+         (log/info "CA already initialized for SSL")
+         (let [{found   true
+                missing false} (group-by fs/exists? required-files)]
+           (if (= required-files missing)
+             (generate-ssl-files! settings keylength)
+             (throw (partial-state-error "CA" found missing))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; certificate_status endpoint
@@ -926,7 +970,6 @@
   [{:keys [csrdir] :as settings} :- CaSettings
    subject :- schema/Str]
   (let [csr-path (path-to-cert-request csrdir subject)]
-    ;; TODO PE-5704 validate CSR policies and return an error somehow
     (autosign-certificate-request! subject (utils/pem->csr csr-path) settings)
     (fs/delete csr-path)
     (log/debugf "Removed certificate request for %s at '%s'" subject csr-path)))
