@@ -140,13 +140,16 @@
 
 (defn client-allowed-access?
   "Determines if the client in the request is allowed to access the
-   certificate_status(es) endpoint based on the client whitelist."
+   certificate_status(es) endpoint based on the client whitelist and
+   whether authorization is required."
   [settings context]
-  (if-let [client-cert (get-in context [:request :ssl-client-cert])]
-    (if (client-on-whitelist? settings client-cert)
-      true
-      (do (log-rejection client-cert) false))
-    (log/info "Access to certificate_status rejected; no client certificate found")))
+  (if (get-in settings [:access-control :certificate-status :authorization-required] true)
+    (if-let [client-cert (get-in context [:request :ssl-client-cert])]
+      (if (client-on-whitelist? settings client-cert)
+        true
+        (do (log-rejection client-cert) false))
+      (log/info "Access to certificate_status rejected; no client certificate found"))
+    true))
 
 (liberator/defresource certificate-status
   [subject settings]
@@ -165,15 +168,15 @@
         :revoked
         ;; A signed cert must exist if we are to revoke it.
         (when-not (ca/certificate-exists? settings subject)
-          (conflict (str "Cannot revoke certificate for host " subject
-                         " - no certificate exists on disk.")))
+          (conflict (str "Cannot revoke certificate for host "
+                         subject " without a signed certificate")))
 
         :signed
         (or
           ;; A CSR must exist if we are to sign it.
           (when-not (ca/csr-exists? settings subject)
-            (conflict (str "Cannot sign certificate for host " subject
-                           " - no certificate signing request exists on disk.")))
+            (conflict (str "Cannot sign certificate for host "
+                           subject " without a certificate request")))
 
           ;; And the CSR must be valid.
           (when-let [error-message (ca/validate-csr settings subject)]
