@@ -8,19 +8,19 @@
 (defservice master-service
   [[:WebserverService add-ring-handler]
    [:PuppetServerConfigService get-config]
-   [:RequestHandlerService handle-request]]
+   [:RequestHandlerService handle-request]
+   [:CaService initialize-master-ssl! retrieve-ca-cert!]]
   (init
    [this context]
-   (let [path            ""
-         config          (get-config)
-         master-certname (get-in config [:puppet-server :certname])
-         master-settings (ca/config->master-settings config)
-         ca-settings     (ca/config->ca-settings config)]
+   (core/validate-memory-requirements!)
+   (let [path        ""
+         config      (get-config)
+         certname    (get-in config [:puppet-server :certname])
+         localcacert (get-in config [:puppet-server :localcacert])
+         settings    (ca/config->master-settings config)]
 
-     ; TODO - https://tickets.puppetlabs.com/browse/PE-3929
-     ; The master needs to eventually get these files from the CA server
-     ; via http or git or something.
-     (ca/initialize! ca-settings master-settings master-certname)
+     (retrieve-ca-cert! localcacert)
+     (initialize-master-ssl! settings certname)
 
      (log/info "Master Service adding a ring handler")
      (add-ring-handler
